#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <linux/limits.h>
#include <signal.h>

#define MAIN_PROGRAM
#include "hopman.h"
#include "gui.h"

void printstatus(int status, void *arg);
gboolean callback_inotify( GIOChannel *source, GIOCondition condition,
			   gpointer pData );
gboolean callback_signal ( GIOChannel *source, GIOCondition condition,
			   gpointer pData );
		       
void callback_hide( GtkWidget *widget, GdkEvent *event, gpointer *pData );
		       
/* Root of the device files tree: */
const char *watchdir="/dev";

/*================================== MAIN ===================================*/
int main(int argc, char **argv)
{
  static int inotfd, sfd, maxfd;
  static GIOChannel *ginotfd, *gsfd;
  static guint rc;
  static time_t interval;
  
  /*-------------- make a global pointer to basename of argv[0] -------------*/
  {
    int i, l;
    l = strlen(argv[0]);
    for(i=l-1 ; argv[0][i] != '/' && i>0 ; i--) ;    /* find last '/' */
    if(argv[0][i] == '/') i++;
    progname = argv[0]+i;
  }

  /*---------------- Record printstatus() as on_exit function ---------------*/
  on_exit(printstatus, NULL);

  /*-------------------------------------------------------------------------*/
  /*          Obtain username and home, read configuration file,             */
  /*            kill previous instance and rewrite pid file.                 */
  do_all_config();

  /*---------------------- get some paramaters ------------------------------*/
  showflags = config_show();
  interval = config_int(MountCheckPeriod);

  /*------------------------- Initialize GTK --------------------------------*/
  gtk_init(&argc, &argv);
  
  /* create window */
  pWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  set_icon (GTK_WINDOW(pWindow));
  /* icon shoule be of type GdkPixbuf * */
  /* gtk_window_set_position(GTK_WINDOW(pWindow), GTK_WIN_POS_NONE ); */
  /* make the window resizable - it also makes it movable */
  gtk_window_set_resizable(GTK_WINDOW(pWindow), TRUE);
  gtk_window_set_title(GTK_WINDOW(pWindow), "Hopman");
  /* set a minimum size so that the title is readable */
  gtk_window_set_default_size(GTK_WINDOW(pWindow), 200, 50);

  /* Make the window sticky if requested */
  if( config_bool(Sticky) == 1 ) gtk_window_stick( GTK_WINDOW(pWindow) );

  /*--------------------- connect the destroy signal ------------------------*/
#if defined(DO_ICONIFY)
  /*g_signal_connect( G_OBJECT(pWindow), "destroy",
		    G_CALLBACK(callback_iconify), NULL ); */
  g_signal_connect_swapped(G_OBJECT(pWindow), "destroy",
			   G_CALLBACK(gtk_window_iconify), GTK_WINDOW(pWindow));
#else
  g_signal_connect( G_OBJECT(pWindow), "destroy",
		    G_CALLBACK(gtk_main_quit), NULL);
#endif
  /*------------------------ Create the menubar -----------------------------*/
  pList = gtk_menu_bar_new();
  gtk_menu_bar_set_pack_direction(GTK_MENU_BAR(pList), GTK_PACK_DIRECTION_TTB);
  gtk_container_add(GTK_CONTAINER(pWindow), pList);
  
  /*-------------------- Make the watch directory current -------------------*/
  if( chdir(watchdir) )
    {
      perror(watchdir);
      return EXIT_FAILURE;
    }

  /*------------------ Initialize inotify and signalfd ----------------------*/
  inotfd  = inotinit();
  ginotfd = g_io_channel_unix_new( inotfd );
  sfd     = signalinit();
  gsfd    = g_io_channel_unix_new( sfd );

  /*----------------------- Declare input watches ---------------------------*/
  rc = g_io_add_watch(ginotfd, G_IO_IN, callback_inotify, NULL);
  rc = g_io_add_watch(gsfd,    G_IO_IN, callback_signal,  NULL);

  /*---------------------- Set up periodic reminder -------------------------*/
  if(interval) start_periodic(interval);
  
  /*---------- report all hotplug partitions whic are alredy present --------*/
  init_list();

  /*---------------------------- Iddle loop ---------------------------------*/
  gtk_main();

  return EXIT_SUCCESS;
}

/*============================ Print exit status ============================*/
void printstatus(int status, void *arg)
{
  switch(status)
    {
    case EXIT_SUCCESS:
      fprintf(stderr, "%s terminated successfully.\n", progname);
      break;
    case EXIT_FAILURE:
      fprintf(stderr, "%s failed.\n", progname);
      break;
    default:
      fprintf(stderr, "%s terminated with status=%d\n", progname, status);
    }
}

/*========================= Channel input callbacks =========================*/
gboolean callback_inotify( GIOChannel *source, GIOCondition condition,
			   gpointer pData )
{
  int inotfd;

  inotfd = (int)g_io_channel_unix_get_fd(source);
  inotify_read(inotfd);
  return TRUE;
}

gboolean callback_signal ( GIOChannel *source, GIOCondition condition,
			   gpointer pData )
{
  int sfd;

  sfd = (int)g_io_channel_unix_get_fd(source);
  signal_read( sfd );
  return TRUE;
}

/*========================== Hide callback ===============================*/
void callback_hide( GtkWidget *widget, GdkEvent *event, gpointer *pData )
{
  ui_set_visible(0);
}
		       
/*================== Set the visibility of the root window ==================*/
void ui_set_visible(int visibility)
{
  /* TODO deiconification doesn't work */
  switch(visibility)
    {
    case 0:
      gtk_window_deiconify( GTK_WINDOW(pWindow) );
      gtk_widget_hide_all( pWindow );
      break;
    case 1:
      gtk_window_deiconify( GTK_WINDOW(pWindow) );
      gtk_widget_show_all( pWindow );
      gtk_window_present( GTK_WINDOW(pWindow) );
      break;
    default:
      gtk_window_deiconify( GTK_WINDOW(pWindow) );
      gtk_widget_show_all(pWindow);
    }
}
