/*
 * annex.cpp
 * Copyright (C) 2021 Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the LICENSE file. * 
 */
 
 
#include <unistd.h>
#include <cstdio>

#include <gtkmm.h>
#include <vector>

#include "annex.h"
#include "fpipe.h"


namespace Hopman
{

Annex::Annex(int argc, char **argv, int x, int y)
{
    for(int i=0; i<argc; i++)
        args.push_back(string(argv[i]));
		
    pid_t pid = getpid();	
    char *c = NULL;
    if (asprintf(&c, "%jd", (intmax_t) pid) != -1) {
        mypid = string(c);
        free(c); // cleanup when done.
    } else
        cerr << "Unable to get the pid\n";
	
    x_screen = x;
    y_screen = y;
    watch_dir = "/dev/";
    socket = "/tmp/" + mypid + "_hopman.socket";
    helper_socket = "/tmp/" + mypid + "_hopman_helper.socket";
    worker_socket = "/tmp/" + mypid + "_hopman_worker.socket";
    get_current_path();
    set_pixmaps();
    set_path_to_the_helper();
}

Annex::~Annex()
{	
    remove( socket.c_str() );
    remove( helper_socket.c_str() );
    remove( worker_socket.c_str() );
}
	
// If we launch the application clicking on the applet, the path will be defined in the .desktop file
// In our case Path=/dev
void Annex::get_current_path()
{
    char current_working_dir[1024] = {0}; 
    getcwd(current_working_dir, 1024);	
    current_path = string(current_working_dir);
}

void Annex::set_pixmaps()
{
    string str = "/usr/share/pixmaps/hopman.png";
    Glib::RefPtr<Gio::File> ref_path = Gio::File::create_for_path( str );	
    if ( ref_path->query_exists() ) 
    app_icon = str;  
    else
        cerr << app_icon + " , not found!" << endl;
	
    str = "/usr/share/hopman/icons/usb-eject.png";
    ref_path = Gio::File::create_for_path( str );	
    if ( ref_path->query_exists() ) 
        eject_icon = str;  
    else
        cerr << eject_icon + " , not found!" << endl;
	
    str = "/usr/share/hopman/icons/usb-removed.png";
    ref_path = Gio::File::create_for_path( str );	
    if ( ref_path->query_exists() ) 
        removed_icon = str;  
    else
        cerr << removed_icon + " , not found!" << endl;
	
    str = "/usr/share/hopman/icons/warning.png";
    ref_path = Gio::File::create_for_path( str );	
    if ( ref_path->query_exists() ) 
        warning_icon = str;  
    else
        cerr << warning_icon + " , not found!" << endl;
}

void Annex::set_path_to_the_helper()
{
    string path = current_path + "/hopman-helper";	
    Glib::RefPtr<Gio::File> ref_path = Gio::File::create_for_path( path );	
    if ( ref_path->query_exists() ) 
        path_to_the_helper = path;  
    else {
        unique_ptr<Hopman::fPipe> fp( new Hopman::fPipe() );
        vector<string> output=fp->get_output( "dpkg-architecture -qDEB_HOST_MULTIARCH" );
        path_to_the_helper = "/usr/lib/" + output.at( output.size()-1 ) + "/hopman/hopman-helper";
        output.clear();
        output.shrink_to_fit();
    }
}

} // namespace Hopman
