/*
 * blk_node.cpp
 * Copyright (C) 2021 Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the LICENSE file. * 
 */


#include "blk_node.h"

#include <string.h>

namespace Hopman {

BlkNode::BlkNode( struct vdev_usb_device device )
{
    pathname=string( device.sysfs_path );
    basename=getBaseName();
    devname="/dev/" + basename; // It should be annex.watch_dir + basename
    mtpt=string( device.mtpt_str );
    vendor=string( device.vendor_str );
    model=string( device.model_str );
    devtype=string( device.devtype_str );
    inode=string( device.inode_str );
    size=string( device.size_str );
}

BlkNode::~BlkNode()
{

}

string BlkNode::getBaseName()
{
    char sep = '/';
    size_t i = pathname.rfind(sep, pathname.length());
    if (i != string::npos) {
        return(pathname.substr(i+1, pathname.length() - i));
    }

    return("");
}

string BlkNode::getDirName()
{
    size_t pos = 0;
    string token;
    string delimiter = "/";
    while ((pos = pathname.find(delimiter)) != string::npos) {
        token = pathname.substr(0, pos);
        pathname.erase(0, pos + delimiter.length());
    }

    return token;
}

bool BlkNode::get_removed()
{
    return this->removed;
}

bool BlkNode::get_removable()
{
    return this->removable;
}

void BlkNode::set_removed(bool value)
{
    this->removed = value;
}

void BlkNode::set_removable(bool value)
{
    this->removable = value;
}

string BlkNode::get_inode()
{
    return this->inode;
}

string BlkNode::get_devname()
{
    return this->devname;
}

string BlkNode::get_basename()
{
    return this->basename;
}

string BlkNode::get_pathname()
{
    return this->pathname;
}

string BlkNode::get_dirname()
{
    return this->dirname;
}

string BlkNode::get_vendor()
{
    return this->vendor;
}

string BlkNode::get_model()
{
    return this->model;
}

string BlkNode::get_devtype()
{
    return this->devtype;
}

string BlkNode::get_size()
{
    return this->size;
}

void BlkNode::set_size (string str)
{
    this->size=str;
}

string BlkNode::get_label()
{
    return this->label;
}

void BlkNode::set_label (string str)
{
    this->label=str;
}

string BlkNode::get_fstype()
{
    return this->fstype;
}

void BlkNode::set_fstype (string str)
{
    this->fstype=str;
}

string BlkNode::get_uuid()
{
    return this->uuid;
}

void BlkNode::set_uuid (string str)
{
    this->uuid=str;
}

string BlkNode::get_partuuid()
{
    return this->partuuid;
}

void BlkNode::set_partuuid (string str)
{
    this->partuuid=str;
}

string BlkNode::get_mtpt()
{
    return this->mtpt;
}

vector<string> BlkNode::get_children_mtpt_vector()
{
    vector<string> v;
    v.begin();
    for(auto &u : children)
        v.push_back(u->get_mtpt());
            return v;
}

} // namespace Hopman
