/*
 * blk_node.h
 * Copyright (C) 2021 Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the LICENSE file. * 
 */


#ifndef __BLK_NODE_H__
#define __BLK_NODE_H__

#include <iostream>
#include <memory>
#include <vector>

extern "C" {

	#include <stat_usb.h>
}

namespace Hopman {

using namespace std;

class BlkNode
{
public:
    BlkNode( struct vdev_usb_device );
    virtual ~BlkNode();

    weak_ptr<BlkNode> parent;
    vector< shared_ptr<BlkNode> > children;
	
    // TODO: make private (setters used by the worker thread)
    void set_removed(bool);
    void set_removable(bool);
    void set_label(string);
    void set_fstype(string);
    void set_size(string);
    void set_uuid(string);
    void set_partuuid(string);

    // Getters
    bool get_removed();
    bool get_removable();
    string get_inode(),
           get_size(),
           get_devname(),
           get_basename(),
           get_dirname(),
           get_pathname(),
           get_label(),
           get_vendor(),
           get_model(),
           get_mtpt(),
           get_fstype(),
           get_devtype(),
           get_uuid(),
           get_partuuid();
			
    vector<string> get_children_mtpt_vector();

private:
    bool removed, removable;
    string devname,
           basename,
           pathname,
           dirname,
           label,
           vendor,
           model,
           mtpt,
           fstype,
           devtype,
           inode,
           size,
           uuid,
           partuuid;

    string getBaseName(),
           getDirName();
};

} // namespace Hopman

#endif //__BLK_NODE_H__
