/*
 * hopman_ui.cpp
 * Copyright (C) 2021 Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the LICENSE file. * 
 */
 
 
#include "hopman_ui.h"
#include "blk_node.h"
#include "blk_tree.h"

namespace Hopman
{

Hopman_Ui::Hopman_Ui(const Hopman::Annex& annex)
: m_Spinner(),
  m_Label(" Scanning... ")
{
    mypid = annex.mypid;
		
    set_border_width(4);
    set_position(Gtk::WIN_POS_MOUSE);
    set_decorated(false);
    set_default_icon_from_file( annex.app_icon );
    set_skip_taskbar_hint();
    set_keep_above();
    present();
	
    add(m_VBox);
    m_VBox.add(m_HBox);
	    
    add_events(Gdk::POINTER_MOTION_MASK);
	
    signal_window_state_event().connect
    (
        sigc::mem_fun(*this, &Hopman_Ui::on_my_window_state_event)
    );
	
    signal_motion_notify_event().connect
    (
        sigc::mem_fun(*this, &Hopman_Ui::on_motion_notify_event)
    );
	
    signal_focus_out_event().connect
	(
        sigc::mem_fun(*this, &Hopman_Ui::on_entry_focus_out_event)
    );

    m_HBox.pack_start(m_Spinner, Gtk::PACK_SHRINK);
    m_HBox.pack_start(m_Label, Gtk::PACK_SHRINK);

    m_Spinner.show();
    m_Spinner.start();	
	
    m_BlkTree=new Hopman::BlkTree( annex );
    m_VBox.pack_start(*m_BlkTree, Gtk::PACK_SHRINK);
	
    show_all_children();
}

Hopman_Ui::~Hopman_Ui()
{
    delete m_BlkTree;
}

bool Hopman_Ui::on_my_window_state_event(GdkEventWindowState* event)
{
    if (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED)
    {
        gtk_main_quit();
    }
		
    return false;
}

bool Hopman_Ui::on_entry_focus_out_event(GdkEventFocus* event)
{
    if (m_BlkTree->get_shall_quit())
    {
        gtk_main_quit();
    }
	
    else return false;
}

bool Hopman_Ui::on_motion_notify_event(GdkEventMotion* event)
{
    /*
     * 
     */

    return true;
}

} // namespace Hopman
