/*
 * menu.cpp
 * Copyright (C) 2021 Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the LICENSE file. * 
 */


#include "hopman_ui.h"
#include "blk_tree.h"
#include "blk_model.h"


namespace Hopman {
    
using namespace std;

BlkTreeCMenu::cMenu(BlkTree *caller)
{
    string label = "Mount";

    item = Gtk::manage(new Gtk::MenuItem(label));
    append(*item);
    
    item->signal_activate().connect
    (
        sigc::mem_fun(*caller, &Hopman::BlkTree::on_mount)
    );
    
    label = "Open in a new tab";
    item = Gtk::manage(new Gtk::MenuItem(label));
    append(*item);
    
    item->signal_activate().connect
    (
        sigc::bind<BlkTree*>
        (
            sigc::mem_fun(*this, &BlkTreeCMenu::on_open_new_tab),
            caller
        )
    );

    label = "Open in a new window";
    item = Gtk::manage(new Gtk::MenuItem(label));
    append(*item);
    
    item->signal_activate().connect
    (
        sigc::bind<BlkTree*>
        (
            sigc::mem_fun(*this, &BlkTreeCMenu::on_open_new_window),
            caller
        )
    );

    show_all_children();
}

BlkTreeCMenu::~cMenu()
{

}

void BlkTreeCMenu::display_menu(GdkEventButton* event, string str, string mountpoint)
{
	this->name = str;
    this->mtpt = mountpoint;
    bool unmounted = true;
    if( this->mtpt.size() ) unmounted = false;
    
    Glib::ListHandle<Gtk::Widget*> childList = this->get_children();
    Glib::ListHandle<Gtk::Widget*>::iterator iter = childList.begin();
    (**iter).set_sensitive(unmounted);
    
    this->popup(event->button, event->time);
}

void BlkTreeCMenu::on_open_new_tab(BlkTree *caller)
{
    if( !this->mtpt.size() ) {
		
        caller->on_mount();
        this->mtpt = caller->get_mountpoint( this->name );
    }
        
    // TODO: use Glib::spawn_async
    GError *error = NULL;
    string cmd = "spacefm --new-tab " + this->mtpt;
    if (!g_spawn_command_line_async(cmd.c_str(), &error)) {
        g_print ("Error running command.\n");
        g_print ("%s\n", error->message);
        g_error_free (error);
    } else {                
        gtk_main_quit();
    }
}

void BlkTreeCMenu::on_open_new_window(BlkTree *caller)
{
    if( !this->mtpt.size() ) {
		
        caller->on_mount();
        this->mtpt = caller->get_mountpoint( this->name );
    }
        
    // TODO: use Glib::spawn_async
    GError *error = NULL;
    string cmd = "spacefm --new-window " + this->mtpt;
    if (!g_spawn_command_line_async(cmd.c_str(), &error)) {
        g_print ("Error running command.\n");
        g_print ("%s\n", error->message);
        g_error_free (error);
    } else {                
        gtk_main_quit();
    }
}
} // namespace Hopman

