/*
 * svr.cpp
 * Copyright (C) 2021 Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the COPYING file. * 
 */


#include <iostream>
#include <string>
#include <vector>
#include "svr.h"

namespace Hopman
{

Svr::Svr(string fileDescriptor)
{
    puntSockServer = (struct sockaddr*) &dirUNIXServer;  
    longServer = sizeof(dirUNIXServer);  
    puntSockClient = (struct sockaddr*) &dirUNIXClient;  
    longClient = sizeof(dirUNIXClient);
	   
    signal(SIGCHLD, SIG_IGN);    /* avoid zombies */
	
    /* Create a bidirectional UNIX socket */
    dfServer = socket ( AF_UNIX, SOCK_STREAM, PROTOCOL );                   
    dirUNIXServer.sun_family = AF_UNIX;   /* domain type */      
    strcpy(dirUNIXServer.sun_path, fileDescriptor.c_str());   /* name of the client */  
    unlink(fileDescriptor.c_str());
}

Svr::~Svr()
{
    if(thread) {
        thread->join();
        thread = nullptr;
    }
        
    close(dfClient);      /* close the file descriptor in the parent */
}

void Svr::dispatch(vector<string> messages)
{
    sigc::slot<void> op_dispatch = sigc::bind< vector<string> >
    (
        sigc::mem_fun(*this, &Svr::send_fileDescriptor),
        messages
    );
				
    thread = Glib::Threads::Thread::create(op_dispatch);
}

void Svr::send_fileDescriptor(vector<string> messages)
{
    /* Create the file descriptor, that is, the socket */ 
    bind(dfServer, puntSockServer, longServer);
                                          
    /* Wait for the request of the client */ 
    listen(dfServer, 5);  
  
    /* Accept the connection */
    dfClient = accept(dfServer, puntSockClient, &longClient);
	
    string number_items=to_string(messages.size());
    write(dfClient, number_items.c_str(), strlen(number_items.c_str()) + 1);

    /* create the child and send the file descriptor */	
    vector<string>::iterator iter;
    iter = messages.begin();
	
    if (fork() == 0) 
        for (; iter < messages.end(); iter++)
            write(dfClient, (*iter).c_str(), strlen((*iter).c_str()) + 1);
}

} // namespace Hopman

                              	


