/*
 * stat_usb - reads usb properties
 * 
 * Forked from systemd/src/udev/udev-builtin-usb_id.c on January 23, 2015.
 * (original from http://cgit.freedesktop.org/systemd/systemd/tree/src/udev/udev-builtin-usb_id.c)
 * 
 * All modifications to the original source file are:
 * Copyright (C) 2015 Jude Nelson <judecn@gmail.com>
 * Copyright (C) 2021 Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 *
 * Original copyright and license text produced below.
 */

/*
 * USB device properties and persistent device path
 *
 * Copyright (c) 2005 SUSE Linux Products GmbH, Germany
 *   Author: Hannes Reinecke <hare@suse.de>
 *
 * Copyright (C) 2005-2011 Kay Sievers <kay@vrfy.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __STAT_USB_H__
#define __STAT_USB_H__

/* Make this header file easier to include in C++ code */
#ifdef __cplusplus
extern "C" {
#endif


#define _BSD_SOURCE
#define _POSIX_C_SOURCE 200809L

#include "common.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <math.h>

#define USB_DT_DEVICE                   0x01
#define USB_DT_INTERFACE                0x04

struct vdev_usb_device {
	
   ino_t inode;
   double size;
   char inode_str[256];
   char size_str[256];
   char mtpt_str[256];
   char vendor_str[256];
   char vendor_str_enc[256];
   char vendor_id[256];
   char model_str[256];
   char model_str_enc[256];
   char product_id[256];
   char serial_str[256];
   char packed_if_str[256];
   char revision_str[256];
   char type_str[256];
   char devtype_str[256];
   char instance_str[256];
   char driver[256];
   char ifnum[256];
   char mayor_minor[256];
   char serial[4096];
   char sysfs_path[8193];
   
};

struct vdev_usb_device_list {
	
  struct vdev_usb_device device;
  struct vdev_usb_device_list *next_ptr;
};

void vdev_usb_device_init ( struct vdev_usb_device* );
static void set_usb_iftype( char*, int, size_t );
static int set_usb_mass_storage_ifsubtype( char*, const char*, size_t );
static void set_scsi_type( char*, const char*, size_t );
static int dev_if_packed_info( char const*, char*, size_t );
int get_device_type( char const*, char**, size_t* );
int get_device_mayor_minor( char const*, char* );
int get_device_size( char const*, double* );

struct vdev_usb_device_list *vdev_usb_device_add( struct vdev_usb_device_list *, char const*, char const*, ino_t );
struct vdev_usb_device_list *new_node( void );
int get_length( struct vdev_usb_device_list* );
int vdev_usb_device_sort( struct vdev_usb_device, struct vdev_usb_device );
struct vdev_usb_device_list *vdev_usb_device_prepend( struct vdev_usb_device_list*, struct vdev_usb_device );
struct vdev_usb_device_list *vdev_usb_device_append( struct vdev_usb_device_list*, struct vdev_usb_device );
struct vdev_usb_device_list *vdev_usb_device_insert(
	struct vdev_usb_device_list*, 
	struct vdev_usb_device, 
	int (*f)(struct vdev_usb_device, struct vdev_usb_device) );
struct vdev_usb_device_list *vdev_usb_device_remove(
	struct vdev_usb_device_list*, 
	struct vdev_usb_device, 
	int (*f)(struct vdev_usb_device, struct vdev_usb_device) );
struct vdev_usb_device_list *vdev_usb_device_clear( struct vdev_usb_device_list* );

#ifdef __cplusplus
}
#endif

#endif /* __STAT_USB_H__ */
